@@ -1,4 +1,4 @@
-use super::{Result, Writer};
+use super::{ObjectError, Result, Writer};
 use crate::{crypto::Digest, ChunkPointer};
 
 use flume as mpsc;
@@ -15,7 +15,9 @@ impl<W: 'static + Writer + Clone> RoundRobinBalancer<W> {
         let (enqueue, dequeue) = mpsc::bounded(writers);
 
         for _ in 0..writers {
-            enqueue.send(writer.clone()).unwrap();
+            enqueue
+                .send(writer.clone())
+                .map_err(|_| ObjectError::Fatal)?;
         }
 
         Ok(RoundRobinBalancer {
@@ -28,18 +30,18 @@ impl<W: 'static + Writer + Clone> RoundRobinBalancer<W> {
 
 impl<W: 'static + Writer> Writer for RoundRobinBalancer<W> {
     fn write_chunk(&mut self, hash: &Digest, data: &[u8]) -> Result<ChunkPointer> {
-        let mut writer = self.dequeue.recv().unwrap();
+        let mut writer = self.dequeue.recv().map_err(|_| ObjectError::Fatal)?;
         let result = writer.write_chunk(hash, data);
-        self.enqueue.send(writer).unwrap();
+        self.enqueue.send(writer).map_err(|_| ObjectError::Fatal)?;
 
         result
     }
 
     fn flush(&mut self) -> Result<()> {
         for _ in 0..self.writers {
-            let mut writer = self.dequeue.recv().unwrap();
+            let mut writer = self.dequeue.recv().map_err(|_| ObjectError::Fatal)?;
             writer.flush()?;
-            self.enqueue.send(writer).unwrap();
+            self.enqueue.send(writer).map_err(|_| ObjectError::Fatal)?;
         }
 
         Ok(())
